#ifndef __DBUS_PROXY_CONEXUS_DBUS_ENDPOINT_H
#define __DBUS_PROXY_CONEXUS_DBUS_ENDPOINT_H

#include <dbus-cxx.h>

#include <conexus/endpoint.h>

#ifndef __DBUS_CONEXUS__STARTSTOP_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__STARTSTOP_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::StartStop , int32_t )
#endif


namespace Conexus {
  namespace DBus {

    class EndpointProxy : public ::DBus::ObjectProxy
    {
      protected:
        EndpointProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.Endpoint", const std::string& path="/Conexus/Endpoint"):
          ::DBus::ObjectProxy(conn, dest, path)
        {
          ::DBus::Path child_path;

          m_method_start_v = this->create_method< void >( "Conexus.Endpoint", "start" );
          m_method_stop_v = this->create_method< void >( "Conexus.Endpoint", "stop" );
          m_method_is_running_b = this->create_method< bool >( "Conexus.Endpoint", "is_running" );
          m_method_change_state_vx = this->create_method< void,int64_t >( "Conexus.Endpoint", "change_state" );
          m_signal_data_received_size = this->create_signal<void,uint64_t>( "Conexus.Endpoint", "data_received_size" );
          m_signal_data_transmitted_size = this->create_signal<void,uint64_t>( "Conexus.Endpoint", "data_transmitted_size" );
          m_signal_opened = this->create_signal<void>( "Conexus.Endpoint", "opened" );
          m_signal_closed = this->create_signal<void>( "Conexus.Endpoint", "closed" );
          m_signal_start_stop = this->create_signal<void,Conexus::StartStop>( "Conexus.Endpoint", "start_stop" );
        }

      public:

        typedef DBusCxxPointer<EndpointProxy> pointer;

        static pointer create( ::DBus::Connection::pointer conn,
                               const std::string& dest="Conexus.Endpoint",
                               const std::string& path="/Conexus/Endpoint"
                             )
        { return pointer( new EndpointProxy(conn, dest, path)); }


        void start( ) { return (*m_method_start_v)(); }

        void stop( ) { return (*m_method_stop_v)(); }

        bool is_running( ) { return (*m_method_is_running_b)(); }

        void change_state( int64_t new_state ) { return (*m_method_change_state_vx)( new_state); }

        ::DBus::signal_proxy<void,uint64_t >& signal_data_received_size() { return *m_signal_data_received_size; }

        ::DBus::signal_proxy<void,uint64_t >& signal_data_transmitted_size() { return *m_signal_data_transmitted_size; }

        ::DBus::signal_proxy<void >& signal_opened() { return *m_signal_opened; }

        ::DBus::signal_proxy<void >& signal_closed() { return *m_signal_closed; }

        ::DBus::signal_proxy<void,Conexus::StartStop >& signal_start_stop() { return *m_signal_start_stop; }


      protected:


        ::DBus::MethodProxy<void>::pointer m_method_start_v;
        ::DBus::MethodProxy<void>::pointer m_method_stop_v;
        ::DBus::MethodProxy<bool>::pointer m_method_is_running_b;
        ::DBus::MethodProxy<void,int64_t>::pointer m_method_change_state_vx;
        ::DBus::signal_proxy<void,uint64_t>::pointer m_signal_data_received_size;
        ::DBus::signal_proxy<void,uint64_t>::pointer m_signal_data_transmitted_size;
        ::DBus::signal_proxy<void>::pointer m_signal_opened;
        ::DBus::signal_proxy<void>::pointer m_signal_closed;
        ::DBus::signal_proxy<void,Conexus::StartStop>::pointer m_signal_start_stop;


    };

  }
}

#endif
