#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_FILEDESCRIPTOR_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_FILEDESCRIPTOR_H

#include <dbus-cxx.h>

#include <conexus/filedescriptor.h>

#include <conexus-dbus/endpoint_adapter.h>


namespace Conexus {
  namespace DBus {

    class FileDescriptorAdapter : public EndpointAdapter
    {
      protected:
        FileDescriptorAdapter( Conexus::FileDescriptor* adaptee=NULL, const std::string& path="/Conexus/FileDescriptor"):
          EndpointAdapter(adaptee, path),
          m_adaptee(adaptee)
        {

          ::DBus::MethodBase::pointer temp_method;

          temp_method = this->create_method<bool>( "Conexus.FileDescriptor", "is_blocking", sigc::mem_fun(*this, &FileDescriptorAdapter::is_blocking_adapter_stub_b) );
          temp_method->set_arg_name( 0, "status" );
          temp_method = this->create_method<bool,bool>( "Conexus.FileDescriptor", "set_blocking", sigc::mem_fun(*this, &FileDescriptorAdapter::set_blocking_adapter_stub_bb) );
          temp_method->set_arg_name( 0, "new_status" );
          temp_method->set_arg_name( 1, "block" );
        }

      public:

        typedef DBusCxxPointer<FileDescriptorAdapter> pointer;

        static pointer create( Conexus::FileDescriptor* adaptee=NULL, const std::string& path="/Conexus/FileDescriptor")
          { return pointer( new FileDescriptorAdapter(adaptee, path)); }

        Conexus::FileDescriptor* adaptee() { return m_adaptee; }

        void set_adaptee( Conexus::FileDescriptor* adaptee ) {
          m_adaptee = adaptee;
        }

        void set_adaptee( Conexus::FileDescriptor& adaptee ) {
          this->set_adaptee(&adaptee);
        }


      private:

        Conexus::FileDescriptor* m_adaptee;


        void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee(); }

        bool is_blocking_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_blocking(); }
        bool set_blocking_adapter_stub_bb( bool block ) { this->check_adaptee(); return m_adaptee->set_blocking( block); }


    };

  }
}

#endif
