#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_IPV4_IP_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_IPV4_IP_H

#include <dbus-cxx.h>

#include <conexus/ipv4_ip.h>

#include <conexus-dbus/endpoint_adapter.h>

#include <conexus-dbus/ipv4_address_adapter.h>
#include <conexus-dbus/ipv4_address_adapter.h>

namespace Conexus {
  namespace DBus {
    namespace IPv4 {

      class IPAdapter : public EndpointAdapter
      {
        protected:
          IPAdapter( Conexus::IPv4::IP* adaptee=NULL, const std::string& path="/Conexus/IPv4/IP"):
            EndpointAdapter(adaptee, path),
            m_adaptee(adaptee)
          {

            ::DBus::MethodBase::pointer temp_method;

            temp_method = this->create_method<void>( "Conexus.IPv4.IP", "bind", sigc::mem_fun(*this, &IPAdapter::bind_adapter_stub_v) );
            temp_method = this->create_method<void>( "Conexus.IPv4.IP", "connect", sigc::mem_fun(*this, &IPAdapter::connect_adapter_stub_v) );
            temp_method = this->create_method<void,uint32_t,uint16_t>( "Conexus.IPv4.IP", "set_remote_address", sigc::mem_fun(*this, &IPAdapter::set_remote_address_adapter_stub_vuq) );
            temp_method->set_arg_name( 1, "addr" );
            temp_method->set_arg_name( 2, "port" );
            temp_method = this->create_method<void,std::string,uint16_t>( "Conexus.IPv4.IP", "set_remote_address", sigc::mem_fun(*this, &IPAdapter::set_remote_address_adapter_stub_vsq) );
            temp_method->set_arg_name( 1, "addr" );
            temp_method->set_arg_name( 2, "port" );
            temp_method = this->create_method<void>( "Conexus.IPv4.IP", "unset_remote_address", sigc::mem_fun(*this, &IPAdapter::unset_remote_address_adapter_stub_v) );
            temp_method = this->create_method<void,uint32_t,uint16_t>( "Conexus.IPv4.IP", "set_local_address", sigc::mem_fun(*this, &IPAdapter::set_local_address_adapter_stub_vuq) );
            temp_method->set_arg_name( 1, "addr" );
            temp_method->set_arg_name( 2, "port" );
            temp_method = this->create_method<void,std::string,uint16_t>( "Conexus.IPv4.IP", "set_local_address", sigc::mem_fun(*this, &IPAdapter::set_local_address_adapter_stub_vsq) );
            temp_method->set_arg_name( 1, "addr" );
            temp_method->set_arg_name( 2, "port" );

            ::DBus::Path child_path;
            
            child_path = path;
            child_path.append_element("LocalAddress");
            DBUS_CXX_DEBUG( "LocalAddress:constructor: child path = " << child_path);
            m_LocalAddress = AddressAdapter::create(NULL, child_path);            this->add_child("LocalAddress", m_LocalAddress);
            if ( adaptee ) m_LocalAddress->set_adaptee(&(adaptee->local_address()));
            
            child_path = path;
            child_path.append_element("RemoteAddress");
            DBUS_CXX_DEBUG( "RemoteAddress:constructor: child path = " << child_path);
            m_RemoteAddress = AddressAdapter::create(NULL, child_path);            this->add_child("RemoteAddress", m_RemoteAddress);
            if ( adaptee ) m_RemoteAddress->set_adaptee(&(adaptee->remote_address()));
          }

        public:

          typedef DBusCxxPointer<IPAdapter> pointer;

          static pointer create( Conexus::IPv4::IP* adaptee=NULL, const std::string& path="/Conexus/IPv4/IP")
            { return pointer( new IPAdapter(adaptee, path)); }

          Conexus::IPv4::IP* adaptee() { return m_adaptee; }

          void set_adaptee( Conexus::IPv4::IP* adaptee ) {
            m_adaptee = adaptee;
            if ( m_adaptee ) {
              m_LocalAddress->set_adaptee( m_adaptee->local_address() );
              m_RemoteAddress->set_adaptee( m_adaptee->remote_address() );
            }
            else {
              m_LocalAddress->set_adaptee( NULL );
              m_RemoteAddress->set_adaptee( NULL );
            }
          }

          AddressAdapter::pointer local_address() { return m_LocalAddress; }

          AddressAdapter::pointer remote_address() { return m_RemoteAddress; }

          void set_adaptee( Conexus::IPv4::IP& adaptee ) {
            this->set_adaptee(&adaptee);
          }


        private:

          Conexus::IPv4::IP* m_adaptee;

          AddressAdapter::pointer m_LocalAddress;
          AddressAdapter::pointer m_RemoteAddress;

          void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee(); }

          void bind_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->bind(); }
          void connect_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->connect(); }
          void set_remote_address_adapter_stub_vuq( uint32_t addr, uint16_t port ) { this->check_adaptee(); m_adaptee->set_remote_address( addr, port); }
          void set_remote_address_adapter_stub_vsq( std::string addr, uint16_t port ) { this->check_adaptee(); m_adaptee->set_remote_address( addr, port); }
          void unset_remote_address_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->unset_remote_address(); }
          void set_local_address_adapter_stub_vuq( uint32_t addr, uint16_t port ) { this->check_adaptee(); m_adaptee->set_local_address( addr, port); }
          void set_local_address_adapter_stub_vsq( std::string addr, uint16_t port ) { this->check_adaptee(); m_adaptee->set_local_address( addr, port); }


      };

    }
  }
}

#endif
