#ifndef __DBUS_PROXY_CONEXUS_DBUS_SERIAL_H
#define __DBUS_PROXY_CONEXUS_DBUS_SERIAL_H

#include <dbus-cxx.h>

#include <conexus/serial.h>

#include <conexus-dbus/filedescriptor_proxy.h>

#include <conexus/serial_interface.h>
#ifndef __DBUS_CONEXUS__READWRITE_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__READWRITE_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::ReadWrite , int32_t )
#endif


namespace Conexus {
  namespace DBus {

    class SerialProxy : public FileDescriptorProxy, public virtual SerialInterface
    {
      protected:
        SerialProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.Serial", const std::string& path="/Conexus/Serial"):
          FileDescriptorProxy(conn, dest, path)
        {
          ::DBus::Path child_path;

          m_method_open_v = this->create_method< void >( "Conexus.Serial", "open" );
          m_method_open_vs = this->create_method< void,std::string >( "Conexus.Serial", "open" );
          m_method_open_vsi = this->create_method< void,const std::string&,Conexus::ReadWrite >( "Conexus.Serial", "open" );
          m_method_rw_mode_u = this->create_method< uint32_t >( "Conexus.Serial", "rw_mode" );
          m_method_device_s = this->create_method< std::string >( "Conexus.Serial", "device" );
          m_method_set_device_vs = this->create_method< void,const std::string& >( "Conexus.Serial", "set_device" );
          m_method_set_device_vsi = this->create_method< void,const std::string&,Conexus::ReadWrite >( "Conexus.Serial", "set_device" );
          m_signal_device_changed = this->create_signal<void>( "Conexus.Serial", "device_changed" );
        }

      public:

        typedef DBusCxxPointer<SerialProxy> pointer;

        static pointer create( ::DBus::Connection::pointer conn,
                               const std::string& dest="Conexus.Serial",
                               const std::string& path="/Conexus/Serial"
                             )
        { return pointer( new SerialProxy(conn, dest, path)); }


        virtual void open( ) { return (*m_method_open_v)(); }

        virtual void open( std::string device ) { return (*m_method_open_vs)( device); }

        virtual void open( const std::string& device, Conexus::ReadWrite read_write_mode ) { return (*m_method_open_vsi)( device, read_write_mode); }

        virtual uint32_t rw_mode( ) { return (*m_method_rw_mode_u)(); }

        virtual std::string device( ) { return (*m_method_device_s)(); }

        virtual void set_device( const std::string& device ) { return (*m_method_set_device_vs)( device); }

        virtual void set_device( const std::string& device, Conexus::ReadWrite read_write_mode ) { return (*m_method_set_device_vsi)( device, read_write_mode); }

        ::DBus::signal_proxy<void >& signal_device_changed() { return *m_signal_device_changed; }


      protected:


        ::DBus::MethodProxy<void>::pointer m_method_open_v;
        ::DBus::MethodProxy<void,std::string>::pointer m_method_open_vs;
        ::DBus::MethodProxy<void,const std::string&,Conexus::ReadWrite>::pointer m_method_open_vsi;
        ::DBus::MethodProxy<uint32_t>::pointer m_method_rw_mode_u;
        ::DBus::MethodProxy<std::string>::pointer m_method_device_s;
        ::DBus::MethodProxy<void,const std::string&>::pointer m_method_set_device_vs;
        ::DBus::MethodProxy<void,const std::string&,Conexus::ReadWrite>::pointer m_method_set_device_vsi;
        ::DBus::signal_proxy<void>::pointer m_signal_device_changed;


    };

  }
}

#endif
