#ifndef __DBUS_PROXY_CONEXUS_DBUS_SERVER_H
#define __DBUS_PROXY_CONEXUS_DBUS_SERVER_H

#include <dbus-cxx.h>

#include <conexus/server.h>


namespace Conexus {
  namespace DBus {

    class ServerProxy : public ::DBus::ObjectProxy
    {
      protected:
        ServerProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.Server", const std::string& path="/Conexus/Server"):
          ::DBus::ObjectProxy(conn, dest, path)
        {
          ::DBus::Path child_path;

          m_method_start_v = this->create_method< void >( "Conexus.Server", "start" );
          m_method_start_vb = this->create_method< void,bool >( "Conexus.Server", "start" );
          m_method_stop_v = this->create_method< void >( "Conexus.Server", "stop" );
          m_method_set_responsiveness_vx = this->create_method< void,int64_t >( "Conexus.Server", "set_responsiveness" );
          m_method_responsiveness_x = this->create_method< int64_t >( "Conexus.Server", "responsiveness" );
          m_method_is_running_b = this->create_method< bool >( "Conexus.Server", "is_running" );
          m_signal_start_stop = this->create_signal<void,int32_t>( "Conexus.Server", "start_stop" );
        }

      public:

        typedef DBusCxxPointer<ServerProxy> pointer;

        static pointer create( ::DBus::Connection::pointer conn,
                               const std::string& dest="Conexus.Server",
                               const std::string& path="/Conexus/Server"
                             )
        { return pointer( new ServerProxy(conn, dest, path)); }


        void start( ) { return (*m_method_start_v)(); }

        void start( bool use_dispatcher ) { return (*m_method_start_vb)( use_dispatcher); }

        void stop( ) { return (*m_method_stop_v)(); }

        void set_responsiveness( int64_t response_us ) { return (*m_method_set_responsiveness_vx)( response_us); }

        int64_t responsiveness( ) { return (*m_method_responsiveness_x)(); }

        bool is_running( ) { return (*m_method_is_running_b)(); }

        ::DBus::signal_proxy<void,int32_t >& signal_start_stop() { return *m_signal_start_stop; }


      protected:


        ::DBus::MethodProxy<void>::pointer m_method_start_v;
        ::DBus::MethodProxy<void,bool>::pointer m_method_start_vb;
        ::DBus::MethodProxy<void>::pointer m_method_stop_v;
        ::DBus::MethodProxy<void,int64_t>::pointer m_method_set_responsiveness_vx;
        ::DBus::MethodProxy<int64_t>::pointer m_method_responsiveness_x;
        ::DBus::MethodProxy<bool>::pointer m_method_is_running_b;
        ::DBus::signal_proxy<void,int32_t>::pointer m_signal_start_stop;


    };

  }
}

#endif
